from uuid import UUID

from db.repositories import (
    TaskRepository, StatusRepository, CommentRepository,
    UserRepository
)
from exceptions import common as common_exc, http as http_exc
from db.models import Type, Status
from .schemas import TaskListSchema, CommentSchema


class TaskService:
    task_repository = TaskRepository()
    status_repository = StatusRepository()
    comment_repository = CommentRepository()
    user_repository = UserRepository()

    async def get_task_list(self, **kwargs):
        tasks = await self.task_repository.get_list(**kwargs)
        return [TaskListSchema.model_validate(task) for task in tasks]

    async def get_task(self, id: UUID):
        task = await self.task_repository.get(id=id)
        await task.fetch_related('status', 'types')

        status_id = task.status.id if task.status else None
        types_data = [type.id for type in task.types]

        task_data = {
            "id": task.id,
            "title": task.title,
            "description": task.description,
            "status": status_id,
            "types": types_data
        }

        return task_data

    async def create_task(self, **kwargs):
        status_uuid = kwargs.get('status')
        type_uuids = kwargs.get('types', [])

        if status_uuid is None:
            raise http_exc.HTTPBadRequestException(detail="Validation error: status: Value must not be None")
        try:
            status = await self.status_repository.get(status_uuid)

        except common_exc.NotFoundException:
            raise http_exc.HTTPBadRequestException(detail=f"status does not exist")

        task = await self.task_repository.create(
            title=kwargs.get('title'),
            description=kwargs.get('description'),
            status=status
        )

        types = await Type.filter(id__in=type_uuids)
        await task.types.add(*types)

        return await self.get_task(id=task.id)

    async def update_task(self, id: UUID, **kwargs):
        status_uuid = kwargs.get('status')
        type_uuids = kwargs.get('types', [])

        if status_uuid:
            kwargs['status'] = await self.status_repository.get(status_uuid)

        types = kwargs.pop('types', [])

        task = await self.task_repository.update(id, **kwargs)

        if type_uuids:
            types = await Type.filter(id__in=types)
            await task.types.clear()
            await task.types.add(*types)

        return await self.get_task(id=task.id)

    async def delete_task(self, id: UUID):
        return await self.task_repository.delete(id)

    async def get_comments(self, id: UUID):
        task = await self.task_repository.get(id)
        return await task.task_comments.all()

    async def add_comment(self, id: UUID, **kwargs):
        try:
            user = await self.user_repository.get(kwargs.get('user'))

        except common_exc.NotFoundException:
            raise http_exc.HTTPBadRequestException(detail=f"user does not exist")

        task = await self.task_repository.get(id)

        comment = await self.comment_repository.create(
            user=user,
            task=task,
            text=kwargs.get('text')
        )

        comment_data = {
            "id": comment.id,
            "text": comment.text,
            "task_id": comment.task.id,
            "user_id": comment.user.id
        }
        return comment_data

    async def delete_comment(self, id: UUID):
        return await self.comment_repository.delete(id)
